import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.awt.image.ConvolveOp;
import java.awt.image.Kernel;
import java.io.File;
import java.util.Arrays;
import java.util.Observable;

import javax.imageio.ImageIO;

import com.jhlabs.image.SolarizeFilter;


public class Model extends Observable {
	
	private boolean filter_blur, filter_distort, filter_sharpen, filter_contrast, filter_invert, filter_bw, filter_zoom, all_enabled, no_image_yet;
	private boolean gotNewImage;
	private boolean[] filterList, oldFilterList;

	private BufferedImage originalImage;
	private String imageName;
	private BufferedImage filteredImage;

	public Model() {
		filter_blur = false;
		filter_distort = false;
		filter_sharpen = false;
		filter_contrast = false;
		filter_invert = false;
		filter_bw = false;
		filter_zoom = false;
		
		//these arrays save the filter settings.
		//blur, distort, sharpen, contrast, invert, bw, zoom
		filterList = new boolean[]{false,false,false,false,false,false,false};
		oldFilterList = new boolean[]{false,false,false,false,false,false,false};
		
		all_enabled = false;
		no_image_yet = true;
		
		gotNewImage = false;
	}
	
	public boolean noImageYet(){
		return no_image_yet;
	}
	
	/**
	 * notifies observing view about program start
	 */
	public void disableButtons(){
		setChanged();
		notifyObservers();	
	}
	
	/**
	 * Sets a new image.
	 * @param newImage
	 */
	public void setImage(String newImage) {
		imageName = newImage;
		
		try {
			originalImage = ImageIO.read(new File(imageName));
		}
		catch (Exception e){
			
		}
		
		gotNewImage = true;
		no_image_yet = false;
		
		setChanged();
		notifyObservers();
	}
	public Image getImage() {
		return originalImage;
	}
	
	public Image getImage2() {
		//after filteredImage is received, there is no "new" image data
		gotNewImage = false;
		
		//due to the lack of filters...
		filteredImage= originalImage;
		return filteredImage;
	}
	
	public String getName() {
		return imageName;
	}
	
	public boolean isBlur() {
		return filter_blur;
	}

	public boolean isDistort() {
		return filter_distort;
	}

	public boolean isSharpen() {
		return filter_sharpen;
	}

	public boolean isContrast() {
		return filter_contrast;
	}

	public boolean isInvert() {
		return filter_invert;
	}
	
	public boolean isBW() {
		return filter_bw;
	}
	
	public boolean isZoom() {
		return filter_zoom;
	}
	
	public boolean receivedNewImage() {
		return gotNewImage;
	}

	public boolean getAllEnabled() {
		return all_enabled;
	}
	
	/**
	 * Activates or de-activates ALL buttons/filters at the same time.
	 * @param allEnabled Controller says whether all Filters are activated.
	 */
	public void setAllEnabled(boolean allEnabled) {
		this.all_enabled = allEnabled;
		if(this.all_enabled == true)
		{			
			oldFilterList = filterList.clone(); //save filterList in oldFilterList
			enableFilter("filter_blur, filter_distort, filter_sharpen, filter_contrast, filter_invert, filter_bw, filter_zoom"); //turn everything on
		}
		if(this.all_enabled == false)
		{
			//naive way to do it: read from the oldFilterList which Filters were activated before
			String activateThese = "";
			if(oldFilterList[0] == true){activateThese += "filter_blur, ";}
			if(oldFilterList[1] == true){activateThese += "filter_distort, ";}
			if(oldFilterList[2] == true){activateThese += "filter_sharpen, ";}
			if(oldFilterList[3] == true){activateThese += "filter_contrast, ";}
			if(oldFilterList[4] == true){activateThese += "filter_invert, ";}
			if(oldFilterList[5] == true){activateThese += "filter_bw, ";}
			if(oldFilterList[6] == true){activateThese += "filter_zoom, ";}
			disableFilter("filter_blur, filter_distort, filter_sharpen, filter_contrast, filter_invert, filter_bw, filter_zoom");
			enableFilter(activateThese); //activate the filters from before
		}
		setChanged();
		notifyObservers();
	}
	
	/**
	 * Fills the filterList with the current Filter-Settings.
	 */
	public void updateFilterList() {
		filterList[0] = this.filter_blur;
		filterList[1] = this.filter_distort;
		filterList[2] = this.filter_sharpen;
		filterList[3] = this.filter_contrast;
		filterList[4] = this.filter_invert;
		filterList[5] = this.filter_bw;
		filterList[6] = this.filter_zoom;
		//System.out.println("Effects are now:" + Arrays.toString(filterList));
		//System.out.println("Effects were before:" + Arrays.toString(oldFilterList));
		//System.out.println("------");
	}

	
	/**
	 * Enables filters.
	 * @param filter String with all filters to be enabled
	 */
	
	public void enableFilter(String filter) {
		if (filter.contains("filter_blur"))
			this.filter_blur = true;
		if (filter.contains("filter_distort"))
			this.filter_distort = true;
		if (filter.contains("filter_sharpen"))
			this.filter_sharpen = true;
		if (filter.contains("filter_contrast"))
			this.filter_contrast = true;
		if (filter.contains("filter_invert"))
			this.filter_invert = true;
		if (filter.contains("filter_bw"))
			this.filter_bw = true;
		if (filter.contains("filter_zoom"))
			this.filter_zoom = true;
		
		updateFilterList();
		
		gotNewImage = true;
		setChanged();
		notifyObservers();
	}
	
	/**
	 * Disables filters.
	 * @param filter String with all filters to be disabled
	 */
	public void disableFilter(String filter) {
		if (filter.contains("filter_blur"))
			this.filter_blur = false;
		if (filter.contains("filter_distort"))
			this.filter_distort = false;
		if (filter.contains("filter_sharpen"))
			this.filter_sharpen = false;
		if (filter.contains("filter_contrast"))
			this.filter_contrast = false;
		if (filter.contains("filter_invert"))
			this.filter_invert = false;
		if (filter.contains("filter_bw"))
			this.filter_bw = false;
		if (filter.contains("filter_zoom"))
			this.filter_zoom = false;
		
		updateFilterList();
		
		gotNewImage = true;
		setChanged();
		notifyObservers();
	}
	
	/**
	 * Switches filters from active to passive and vice versa
	 * @param filter String with all filters to be switched
	 */
	
	public void switchFilter(String filter) {
		if (filter.contains("filter_blur"))
			this.filter_blur = !this.filter_blur;
		if (filter.contains("filter_distort"))
			this.filter_distort = !this.filter_distort;
		if (filter.contains("filter_sharpen"))
			this.filter_sharpen = !this.filter_sharpen;
		if (filter.contains("filter_contrast"))
			this.filter_contrast = !this.filter_contrast;
		if (filter.contains("filter_invert"))
			this.filter_invert = !this.filter_invert;
		if (filter.contains("filter_bw"))
			this.filter_bw = !this.filter_bw;
		if (filter.contains("filter_zoom"))
			this.filter_zoom = !this.filter_zoom;
		
		updateFilterList();
		
		gotNewImage = true;
		setChanged();
		notifyObservers();
	}
}
